/*
 * gnome-drivers
 * -------------
 * Do a bunch of stuff with drivers (kernel modules) with a pretty
 * GNOME user interface.
 *
 * Based on the module-init-tools code by Rusty
 *
 * All the horrible code (namely my gnome hacks), (C)2004 Stewart Smith
 *
 * Distributed under the GPL.
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <glade/glade.h>

int load_module_data(GtkWidget *store, char *module_name);

GtkWidget *aboutBox;
GladeXML *xml;

enum {
	LOADED_COLUMN,
	MODULE_COLUMN,	/* Module Name */
	SIZE_COLUMN,	/* Module Size */
	USERS_COLUMN,	/* Module Users */
	N_COLUMNS		/* Total number of columns */
};


/* Prototype for selection handler callback */
static void tree_selection_changed_cb (GtkTreeSelection *selection, gpointer data);
void on_getModuleInfoButton_clicked(GtkButton* button, gpointer data);

static void destroyer_free(gpointer data,GClosure *pants)
{
	g_free(data);
}

gchar *selected=NULL;

static void
tree_selection_changed_cb (GtkTreeSelection *selection, gpointer data)
{
        GtkTreeIter iter;
        GtkTreeModel *model;
        gchar *module;

        if (gtk_tree_selection_get_selected (selection, &model, &iter))
        {
                gtk_tree_model_get (model, &iter, MODULE_COLUMN, &module, -1);

                g_print ("You selected module %s\n", module);
				if(selected)
						g_free(selected);
				selected=module;
        }
}

void on_getModuleInfoButton_clicked(GtkButton* button, gpointer data)
{
	GtkWidget *w,*tree;
	g_print("You Want more Information on: %s\n",selected);
	w = glade_xml_get_widget(xml,"modInfoDialog");
	tree = glade_xml_get_widget(xml,"moduleInfoTree");
	gtk_window_set_title(GTK_WINDOW(w),selected);
	GtkTreeStore *store = gtk_tree_store_new (2, 
											  G_TYPE_STRING,   /* Key */
                	                          G_TYPE_STRING); /* Value */
	load_module_data(store, selected);

	gtk_tree_view_set_model(GTK_TREE_VIEW(tree),GTK_TREE_MODEL(store));

	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Key",
                                                   renderer,
                                                   "text", 0,
                                                   NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);

	column = gtk_tree_view_column_new_with_attributes ("Value",
                                                   renderer,
                                                   "text", 1,
                                                   NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
	
	gtk_widget_show(w);
}

void on_about1_activate(GtkWidget *widget, gpointer user_data) {
	gtk_widget_show(aboutBox);
}

void on_about2_close(GtkWidget *widget, gpointer user_data) {
	gtk_widget_hide(widget);
}

void on_quit1_activate(GtkWidget *widget, gpointer user_data) {
	gtk_main_quit();
}

int main (int argc, char *argv[])
{
	GtkWidget *window1;
	GtkWidget *treeView;

	#ifdef ENABLE_NLS
		bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
		textdomain (PACKAGE);
	#endif

	gnome_init (PACKAGE, VERSION, argc, argv);
	glade_gnome_init ();
	/*
	 * The .glade filename should be on the next line.
	 */
	xml = glade_xml_new (PACKAGE_SOURCE_DIR"/Drivers.glade", NULL, NULL);

	/* This is important */
	glade_xml_signal_autoconnect (xml);
	window1 = glade_xml_get_widget (xml, "app1");
	treeView = glade_xml_get_widget(xml, "treeview1");
	aboutBox = glade_xml_get_widget(xml, "about2");
	gtk_widget_show (window1);

	GtkTreeStore *store = gtk_tree_store_new (N_COLUMNS, 
										  G_TYPE_BOOLEAN, /* Is loaded */
                                          G_TYPE_STRING,   /* Module Name */
                                          G_TYPE_STRING,   /* Module Size */
                                          G_TYPE_STRING); /* Module Users */
	char line[4096];
	FILE *file = fopen("/proc/modules", "r");
    if (!file) {
		GtkMessageDialog *dialog = gtk_message_dialog_new(GTK_WINDOW(window1),
       	                          GTK_DIALOG_DESTROY_WITH_PARENT,
                                  GTK_MESSAGE_ERROR,
                                  GTK_BUTTONS_CLOSE,
									"Error Openining /proc/modules");
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);

		exit(1);
	}
	
	while (fgets(line, sizeof(line), file)) {
		GtkTreeIter   iter;
		gtk_tree_store_append (store, &iter, NULL);  /* Acquire an iterator */

		gtk_tree_store_set (store, &iter,
						LOADED_COLUMN, gtk_true,
        	            MODULE_COLUMN,	strtok(line, " \t"),
            	        SIZE_COLUMN, 	strtok(NULL, " \t\n"),
                	    USERS_COLUMN,	strtok(NULL, " \t\n"),
                    	-1);
		
	};
	fclose(file);
	
	gtk_tree_view_set_model(GTK_TREE_VIEW(treeView),GTK_TREE_MODEL(store));

	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	
	renderer = gtk_cell_renderer_toggle_new ();
	column = gtk_tree_view_column_new_with_attributes ("Loaded",
                                                   renderer,
												   "active", LOADED_COLUMN,
                                                   NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeView), column);

	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Module",
                                                   renderer,
                                                   "text", MODULE_COLUMN,
                                                   NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeView), column);

	column = gtk_tree_view_column_new_with_attributes ("Size",
                                                   renderer,
                                                   "text", SIZE_COLUMN,
                                                   NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeView), column);
	
	column = gtk_tree_view_column_new_with_attributes ("Users",
                                                   renderer,
                                                   "text", USERS_COLUMN,
                                                   NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeView), column);
	
	
	GtkWidget *getModuleInfoButton;
	getModuleInfoButton = glade_xml_get_widget(xml,"getModuleInfoButton");
	
	/* Setup the selection handler */
	GtkTreeSelection *select;

	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeView));
	gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
	g_signal_connect(	G_OBJECT (select), "changed",
    	              		G_CALLBACK (tree_selection_changed_cb),
        	          		NULL);
	gtk_main ();
	return 0;
}
